package puppet.ambition;

import info.frantzen.testing.ssmsimulator.types.ComplexType;
import info.frantzen.testing.ssmsimulator.types.ST_Boolean;
import info.frantzen.testing.ssmsimulator.types.ST_PosInt;
import info.frantzen.testing.ssmsimulator.types.ST_PseudoPosDouble;
import info.frantzen.testing.ssmsimulator.types.ST_String;
import info.frantzen.testing.ssmsimulator.types.Type;
import info.frantzen.testing.ssmsimulator.types.TypeInstance;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.text.StyledEditorKit.ItalicAction;
import javax.xml.namespace.QName;

public class TypeConverter {

    private HashMap<String,String> ambitionTypeMapping;

    private HashMap<String,String> ambition2JavaTypeMapping;
    
    private HashMap<String,String> objectNameTypeMapping;
    
    private HashMap<String,String> xsdWrapperTypeMapping;
    
    public TypeConverter (){
        ambitionTypeMapping = new HashMap<String,String>();
        ambition2JavaTypeMapping = new HashMap<String,String>();
        objectNameTypeMapping = new HashMap<String,String>();
        xsdWrapperTypeMapping = new HashMap<String, String>();
        configureAmbitionTypeMapping();
        configureAmbition2JavaBasicTypeMapping();
        configureNameObjectType();
        configureXSDWrapperType();        
    }
    
    private void configureAmbitionTypeMapping (){
        ambitionTypeMapping.put("int", "ST_PosInt");
        ambitionTypeMapping.put("Integer", "ST_PosInt");
        ambitionTypeMapping.put("boolean", "ST_Boolean");
        ambitionTypeMapping.put("Boolean", "ST_Boolean");
        ambitionTypeMapping.put("double", "ST_PseudoPosDouble");
        ambitionTypeMapping.put("Double", "ST_PseudoPosDouble");
        ambitionTypeMapping.put("String", "ST_String");
        ambitionTypeMapping.put("java.lang.String", "ST_String");
        ambitionTypeMapping.put("Enumeration", "Enumeration");    
    } 

    private void configureAmbition2JavaBasicTypeMapping (){
        ambition2JavaTypeMapping.put("ST_PosInt","int");
        ambition2JavaTypeMapping.put("ST_Boolean","boolean");
        ambition2JavaTypeMapping.put("ST_PseudoPosDouble", "double");
        ambition2JavaTypeMapping.put("ST_String","String");
        ambition2JavaTypeMapping.put("Enumeration", "Enumeration");    
    } 

    private void configureNameObjectType(){
        objectNameTypeMapping.put("int", "Integer");
        objectNameTypeMapping.put("boolean", "Boolean");
        objectNameTypeMapping.put("double", "Double");
        objectNameTypeMapping.put("char", "String");
    } 
    
    private void configureXSDWrapperType(){
        xsdWrapperTypeMapping.put("int", "int");
        xsdWrapperTypeMapping.put("boolean", "boolean");
        xsdWrapperTypeMapping.put("double", "double");
        xsdWrapperTypeMapping.put("char", "char");
        xsdWrapperTypeMapping.put("string", "String");
    } 

    public Type getConvertedAmbitionType(String inType){
        Object [][] fields = null;
        
        if (ambitionTypeMapping.containsKey(inType)){
            if (ambitionTypeMapping.get(inType).equals("ST_PosInt"))
                return new ST_PosInt();
            if (ambitionTypeMapping.get(inType).equals("ST_Boolean"))
                return new ST_Boolean();
            if (ambitionTypeMapping.get(inType).equals("ST_PseudoPosDouble"))
                return new ST_PseudoPosDouble();
            if (ambitionTypeMapping.get(inType).equals("ST_String"))
                return new ST_String();                
//            if (typeMapping.get(inType).equals("Enumeration"))
//                return new Enumeration;                
        }
// the case in Ambition it matches with "<T>"
        try {
            Class<?> genericClass = Class.forName(inType);
            Method [] methodList = genericClass.getDeclaredMethods();
            
            ArrayList<Type> fieldTypes = new ArrayList<Type>();
            ArrayList<String> fieldNames =  new ArrayList<String>();
            
            for (int i=0; i<methodList.length; i++){                
                if (Modifier.isPublic(methodList[i].getModifiers()) && !Modifier.isStatic(methodList[i].getModifiers())){
                    
                    System.out.println(genericClass.getName() +": "+methodList[i].getName());
                    System.out.println(genericClass.getName() +": "+methodList[i].getReturnType().getName());

                    if (methodList[i].getName().startsWith("get")){
                        fieldTypes.add( getConvertedAmbitionType(methodList[i].getReturnType().getName()) );
                        fieldNames.add(methodList[i].getName());
                    }
                }
            }

            fields = new Object [fieldNames.size()][2];
            for (int i=0; i<fieldNames.size(); i++){
                fields[fieldNames.size()-1-i][0] = fieldNames.get(i);
                fields[fieldNames.size()-1-i][1] = fieldTypes.get(i);
            }
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        
        ComplexType currentComplexType = new ComplexType(new QName(inType),fields);
        
        return currentComplexType;
    } 
        
    public String getAmbitionType2JavaBasicType(String inType){
        if (ambition2JavaTypeMapping.containsKey(inType))
            return ambition2JavaTypeMapping.get(inType);

// the case in Java it matches with "<T>" 
        return "Object";
    }
    
    public String getNameObjectType(String inType){

        if (objectNameTypeMapping.containsKey(inType))
            return objectNameTypeMapping.get(inType);

// the case in Java it matches with "<T>" 
        return inType;
    } 

    public String getJavaTypeFromXSD(String inType){

        if (xsdWrapperTypeMapping.containsKey(inType))
            return xsdWrapperTypeMapping.get(inType);

// the case in Java it matches with "<T>" 
        return inType;
    } 
    
    public boolean isAnAmbitionComplexType (String inTypeName){
        return  !ambitionTypeMapping.containsKey(inTypeName);
    }
    
//    static public Object[] toArray ( TypeInstance inputInstance){
//        Object [] arrayRepresentation = inputInstance.toString().split(",");
//        return arrayRepresentation;
//    }
}

