/*
 * Service.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on February 7, 2007, 3:35 PM
 *
 */

package info.frantzen.testing.ssmsimulator.ssm;

import java.util.HashSet;
import java.util.Iterator;

/**
 * A service. Every service consists of a set of {@link Port ports}.
 * @author Lars Frantzen
 * @version 0.1
 */
public class Service {
    
    private String name;
    private HashSet<Port> ports = new HashSet<Port>();
    
    /**
     * Creates a new instance of Service.
     * @param name the name of the service
     */
    public Service(String name) {
        this.name = name;
    }
    
    /**
     * Getter for the name attribute.
     * @return the name of the service
     */
    public String getName() {
        return name;
    }
    
    /**
     * Adds a port to the service.
     * @param p the port to be added
     */
    public void addPort(Port p) {
        ports.add(p);
    }
    
    /**
     * Getter for the set of ports of the service.
     * @return the set of ports of the SSM
     */
    public HashSet<Port> getPorts() {
        return ports;
    }
    
    /**
     * A String representation of the Service.
     * @return a String representation of the Port
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("Service " + name + '\n');
        sb.append(ports);
        return sb.toString();
    }
    
    /**
     * Checks the consistency, i.e. the name is not the empty String and there is at least one port.
     * @return true if the consistency is ok, false otherwise
     */
    public boolean checkConsistency() {
        return ((name != "") && (ports.size() != 0));
    }
}
