/*
 * ST_Boolean.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 30. Dezember 2006, 13:30
 *
 */

package info.frantzen.testing.ssmsimulator.types;

import javax.xml.namespace.QName;

/**
 * The (simple) Boolean type.
 * @author Lars Frantzen
 * @version 0.3
 */
public class ST_Boolean extends SimpleType {
    
    Boolean nextBoolean = new Boolean(false);
    
    /**
     * Creates a new instance of ST_Boolean.
     */
    public ST_Boolean() {
        super(new QName("ST_Boolean"), "Boolean");
    }
    
    /**
     * Returns a random {@link ST_BooleanInstance ST_Boolean instance}, i.e. either a true
     * or a false.
     * @return a random ST_Boolean instance
     */
    public ST_BooleanInstance generateRandomInstance() {
        return new ST_BooleanInstance(new Boolean(random.nextBoolean()));
    }
    
    /**
     * Returns the next boundary {@link ST_BooleanInstance ST_Boolean instance}, i.e. alternating true
     * and false. Initially true is returned.
     * @return the next boundary ST_Boolean instance
     */
    public ST_BooleanInstance generateNextBoundaryInstance() {
        nextBoolean = new Boolean(!(nextBoolean.booleanValue()));
        return new ST_BooleanInstance(nextBoolean);
    }
    
    /**
     * Returns the number of boundary {@link ST_BooleanInstance ST_Boolean instances},
     * i.e. 2 (true and false).
     * @return the number of boundary ST_Boolean instances, i.e. 2
     */
    public int numberOfBoundaryInstances() {
        return 2;
    }
    
    /**
     * Returns the initial {@link ST_BooleanInstance ST_Boolean instance}, i.e. false.
     * @return the initial ST_Boolean instance, i.e. false
     */
    public ST_BooleanInstance generateInitialInstance() {
        return new ST_BooleanInstance(new Boolean(false));
    }
    
    /**
     * Returns true for all Boolean objects since this simple type is not restricted.
     * @param b the object for which the restriction shall be verified
     * @return true for all Boolean instances
     */
    public boolean checkRestriction(Object b) {
        if (b instanceof Boolean)
            return true;
        else
            return false;
    }
    
    /**
     * A string representation of the type.
     * @return a string representation of the type
     */
    public String toString() {
        return "ST_Boolean";
    }

    public static TypeInstance instanceFromNumber(Integer number) {
        if (number == 0)
            return new ST_BooleanInstance(false);
        else
            return new ST_BooleanInstance(true);
    }

}
