/*
 * ST_PseudoPosDoubleInstance.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on May 6 2007
 *
 */

package info.frantzen.testing.ssmsimulator.types;

/**
 * An ST_PseudoPosDoubleInstance is an instance of an {@link ST_PseudoPosDouble}.
 * @author Lars Frantzen
 * @version 0.1
 */
public class ST_PseudoPosDoubleInstance extends SimpleTypeInstance {
    
    public ST_PseudoPosDoubleInstance(int prePointValue, int postPointValue) {
        super(null);
        
        // System.err.println("XXXXX " + postPointValue);
        
        int[] value = new int[2];
        value[0] = prePointValue;
        value[1] = postPointValue;
        super.setValue(value);
    }
    
    public ST_PseudoPosDoubleInstance(int[] value) {
        super(value);
    }
    
    public ST_PseudoPosDoubleInstance(double value) {
        super(null);
        setValue(value);
    }
    
    /**
     * A string representation of the ST_PseudoPosDoubleInstance.
     * @return a string representation of the ST_PosDoubleInstance
     */
    public String toString() {
        // int[] value = (int[]) getValue();
        // return Integer.toString(value[0]) + '.' + Integer.toString(value[1]);
        return Double.toString((Double) getValue());
    }
    
    /**
     * Clones this ST_PseudoPosDoubleInstance.
     * @return the cloned ST_PseudoPosDoubleInstance
     */
    public ST_PseudoPosDoubleInstance clone() {
        return new ST_PseudoPosDoubleInstance(((int[]) super.getValue()).clone());
    }
    
    public void setValue(Object o) {
        Double d = (Double) o;
        
        // System.err.println("Transforming " + d);
        
        int[] value = new int[2];
        value[0] = d.intValue();
        String s = d.toString();
        value[1] = ST_PseudoPosDouble.stringAsPostInt(s.substring(s.indexOf('.')+1,s.length()));
        
        // System.err.println("to " + value[0] + ',' + value[1]);
        
        super.setValue(value);
        // System.err.println("As String: " + toString());
        
    }
    
    public Object getValue() {
        int[] value = (int[]) super.getValue();
        String s = Integer.toString(value[0]) + '.' + ST_PseudoPosDouble.postIntAsString(value[1]);
        return Double.valueOf(s);
    }
    
    
    public int asFlatInteger() {
        int[] value = (int[]) super.getValue();
        int leftFlat = value[0] * (int) Math.pow(10,ST_PseudoPosDouble.postPointLength);
        
        return leftFlat + value[1];
    }
    
    
}
