/*
 * TypeInstance.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 30. Dezember 2006, 14:37
 *
 */

package info.frantzen.testing.ssmsimulator.types;

/**
 * This is the abstract root class of the type instances, i.e. objects representing
 * values of the {@link Type types}. There are three kinds of type instances:<br>
 * - {@link EnumerationInstance enumeration instances} -  a single identifier (out of a list of string identifiers)<br>
 * - {@link SimpleTypeInstance simple type instances} - an object which corresponds to a java primitive type<br>
 * - {@link ComplexTypeInstance complex type instances} - an object similar to an XML-Schema complex type instance, or what is
 * called a <I>record</I> in programming languages<br>
 * <br>
 * The main purpose of these classes is to type the objects and to enforce the
 * {@link #clone() clone} method.
 * @author Lars Frantzen
 * @version 0.1
 */
public abstract class TypeInstance {
    
    /**
     * Every type instance class must be able to create clones.
     * @return the cloned type instance
     */
    public abstract TypeInstance clone();
    
    private Object value;
    
    /**
     * Getter for the value which is represented by this type instance.
     * @return the value which is represented by this type instance
     */
    public Object getValue() {
        return value;
    }
    
    /**
     * Setter for the value which is represented by this type instance.
     * @param o the value which is represented by this type instance
     */
    public void setValue(Object o) {
        value = o;
    }
    
    /**
     * Creates a new instance of TypeInstance.
     * @param value the object which is represented by this type instance
     */
    public TypeInstance(Object value) {
        this.value = value;
    }
    
}
